"""process_output.py
by warnuk

This module forms a pandas dataframe using the hourly data from PVWatts. It calculates daily summaries and max/min/med
days from the daily data."""

import pandas as pd
from PVWatts_Tool import dataframe_template

__author__ = "warnuk"
__credits__ = ["warnuk", "NREL", "PVWatts"]
__license__ = "MIT"
__version__ = "1.0.1"
__maintainer__ = "warnuk"
__email__ = "warnuk@umich.edu"
__status__ = "Development"

def populate_df(data):
    """put data into pandas DataFrame using template from dataframe_template module"""
    df = dataframe_template.template
    ac_kw = []
    for value in data['ac']:
        ac_kw.append(value / 1000)
    df['AC (kW)'] = ac_kw
    return (df)


def kW_per_day(df):
    """sum hourly data by day;
    only call after populate_df is called!"""
    summed_by_day = []
    for month in range(1, 13):
        profile = df.loc[df['Month'] == month, :]
        for day in range(1, 32):
            power = profile.loc[profile['Day'] == day, :]
            generation = power['AC (kW)'].sum()
            if generation:
                summed_by_day.append([month, day, generation])
    return (pd.DataFrame(summed_by_day, columns=['Month', 'Day', 'AC (kW)']))


def peak_days(df):
    """select max, min, and med days);
    only call after populate_df AND kW_per_day are called!"""
    max_day = df.loc[df['AC (kW)'] == df['AC (kW)'].max(), :]
    min_day = df.loc[df['AC (kW)'] == df['AC (kW)'].min(), :]
    med_day = df.loc[df['AC (kW)'] == df['AC (kW)'].median(), :]
    return ({'max': max_day, 'min': min_day, 'median': med_day})



"""PVWAtts_API.py
by warnuk

This module uses the request functions from the nrel_requests module to build an object for each scenario and
assign that object attributes which are called to deliver output.

The PVWatts_Run object collects the necessary parameters to make a request to the PVWatts API as attributes.
Hourly output, annual production, max/min/median days, and rate information are stored as attributes as well."""

from PVWatts_Tool import nrel_requests
from PVWatts_Tool import process_output

__author__ = "warnuk"
__credits__ = ["warnuk", "NREL", "PVWatts"]
__license__ = "MIT"
__version__ = "1.0.1"
__maintainer__ = "warnuk"
__email__ = "warnuk@umich.edu"
__status__ = "Development"
datetime_reference = "datetime_defaults.csv"

class PVWatts_Run(object):
    def __init__(self, api_key, area, module_type, lat, lon, losses,
                 array_type, tilt, azimuth, timeframe, ratetype):
        """PVWatts_Run object takes location/system attributes to
        call PVWatts API and assign output to its output attribute"""
        self.api_key = api_key
        self.area = area
        self.module_type = module_type
        self.lat = lat
        self.lon = lon
        self.losses = losses
        self.array_type = array_type
        self.tilt = tilt
        self.azimuth = azimuth
        self.timeframe = timeframe
        self.ratetype = ratetype

        # Use proper efficiency rating for module_type.
        if self.module_type == 0:
            self.system_capacity = round(0.15 * self.area, 1)
        elif self.module_type == 1:
            self.system_capacity = round(0.19 * self.area, 1)
        elif self.module_type == 2:
            self.system_capacity = round(0.1 * self.area, 1)

        self.pvwatts_output = nrel_requests.pvwatts_output(api_key=self.api_key,
                                                           system_capacity=self.system_capacity,
                                                           module_type=self.module_type,
                                                           lat=self.lat,
                                                           lon=self.lon,
                                                           losses=self.losses,
                                                           array_type=self.array_type,
                                                           tilt=self.tilt,
                                                           azimuth=self.azimuth,
                                                           timeframe=self.timeframe)

        self.utility_output = nrel_requests.utility_output(lat=self.lat, lon=self.lon, api_key=self.api_key)

        self.util_name = self.utility_output['utility_name']
        self.rate = self.utility_output[self.ratetype]

        self.ac_annual = self.pvwatts_output['ac_annual']

        self.energy_value = round(self.ac_annual * self.rate, 2)

        self.hourly_data = process_output.populate_df(self.pvwatts_output)

        self.daily_data = process_output.kW_per_day(self.hourly_data)

        self.max = process_output.peak_days(self.daily_data)['max']
        self.max_ratio = self.max.iloc[0, 2] / self.area

        self.min = process_output.peak_days(self.daily_data)['min']
        self.min_ratio = self.min.iloc[0, 2] / self.area

        self.median = process_output.peak_days(self.daily_data)['median']
        self.median_ratio = self.median.iloc[0, 2] / self.area

        self.annual_ratio = self.pvwatts_output['ac_annual'] / self.area

    def describe(self):
        return (f"System Capacity: {self.system_capacity} (kWdc)"
                f"\n\nPeak Day (max) Ratio: {self.max_ratio} (kWh/m^2/day)"
                f"\nLow Day (min) Ratio: {self.min_ratio} (kWh/m^2/day)"
                f"\nAvg. Day (median) Ratio: {self.median_ratio} (kWh/m^2/day)"
                f"\nAnnual Ratio: {self.annual_ratio} (kWh/m^2/yr)"
                f"\n\nAnnual AC Solar Potential: {self.ac_annual} (kWh)"
                f"\nUtility: {self.util_name}"
                f"\nRate: ${self.rate}/kWh"
                f"\nEnergy Value: ${self.energy_value}")



"""dataframe_template.py
by warnuk

This module sets up a template pandas DataFrame that will be used to sort hourly data from PVWatts into daily data."""

import pandas as pd

__author__ = "warnuk"
__credits__ = ["warnuk", "NREL", "PVWatts"]
__license__ = "MIT"
__version__ = "1.0.1"
__maintainer__ = "warnuk"
__email__ = "warnuk@umich.edu"
__status__ = "Development"

# Create a dictionary representing the typical number of days in a month.
days = {1: 31,
        2: 28,
        3: 31,
        4: 30,
        5: 31,
        6: 30,
        7: 31,
        8: 31,
        9: 30,
        10: 31,
        11: 30,
        12: 31}

# Create an empty list to contain the month number for each row in the final DataFrame.
Month = []

# Iterate through each 'month' in the days dictionary to populate the Month list with the correct Month IDs.
for i in days:
    for j in range(0, days[i]):
        for x in range(0, 24):
            Month.append(i)
# Create an empty list to contain the day-of-the-month for each row in the final DataFrame.
Day = []

# Iterate through each 'day' in the days dictionary to populate the Day list with the correct Day IDs.
for i in days:
    for x in range(1, days[i] + 1):
        for hour in range(0, 24):
            Day.append(x)

# Create an empty list to contain the hour for each row in the final DataFrame.
Hour = []

# Iterate through each day-of-the-year and each hour-of-the-day to populate the Hour list with the correct Hour IDs.
for i in range(0, 365):
    for x in range(0, 24):
        Hour.append(x)

# Create the final DataFrame to be used as a template by the process_output module.
cols = ["Month", "Day", "Hour"]
template = pd.DataFrame({"Month": Month,
                         "Day": Day,
                         "Hour": Hour})[cols]





#! /usr/bin/env python
import sys
import os
import pandas
from PyQt5 import QtWidgets

from PVWatts_Tool import PVWatts_API

__author__ = "warnuk"
__credits__ = ["warnuk", "NREL", "PVWatts"]
__license__ = "MIT"
__version__ = "1.0.1"
__maintainer__ = "warnuk"
__email__ = "warnuk@umich.edu"
__status__ = "Development"

class Window(QtWidgets.QWidget):
    def __init__(self):
        super().__init__()
        self.init_ui()

    def init_ui(self):
        """ Initialize the Graphical User Interface.

        The GUI consists of a main container, a parameter pane (with labels and fields), and an output pane.
        The submit button creates a new instance of a pvwatts_run object. The pvwatts_run object calls NREL APIs using
        user-input to generate solar potential estimates and rate/utility information. This info is then output to the
        output box on the right side of the container."""

        self.first_run = True
        container = QtWidgets.QHBoxLayout()

        left_pane = QtWidgets.QVBoxLayout()
        right_pane = QtWidgets.QVBoxLayout()

        parameters = QtWidgets.QHBoxLayout()
        param_labels = QtWidgets.QVBoxLayout()
        param_fields = QtWidgets.QVBoxLayout()

        self.api_l = QtWidgets.QLabel("API Key: ")
        self.area_l = QtWidgets.QLabel("Area (m^2): ")
        self.module_l = QtWidgets.QLabel("Module Type: ")
        self.lat_l = QtWidgets.QLabel("Lat (+/- °): ")
        self.lon_l = QtWidgets.QLabel("Lon (+/- °): ")
        self.losses_l = QtWidgets.QLabel("Losses (%): ")
        self.array_l = QtWidgets.QLabel("Array Type: ")
        self.tilt_l = QtWidgets.QLabel("Tilt (°)")
        self.azimuth_l = QtWidgets.QLabel("Azimuth (°): ")
        self.rate_l = QtWidgets.QLabel("Rate Type: ")

        param_labels.addWidget(self.api_l)
        param_labels.addWidget(self.area_l)
        param_labels.addWidget(self.module_l)
        param_labels.addWidget(self.lat_l)
        param_labels.addWidget(self.lon_l)
        param_labels.addWidget(self.losses_l)
        param_labels.addWidget(self.array_l)
        param_labels.addWidget(self.tilt_l)
        param_labels.addWidget(self.azimuth_l)
        param_labels.addWidget(self.rate_l)

        self.api_le = QtWidgets.QLineEdit()
        self.area_le = QtWidgets.QLineEdit("1000")
        self.module_le = QtWidgets.QComboBox()
        self.module_le.insertItems(0, ["Standard", "Premium", "Thin Film"])
        self.lat_le = QtWidgets.QLineEdit("42.3")
        self.lon_le = QtWidgets.QLineEdit("-83.7")
        self.losses_le = QtWidgets.QLineEdit("14")
        self.array_le = QtWidgets.QComboBox()
        self.array_le.insertItems(0, ["Fixed - Open Rack", "Fixed - Roof Mounted", "1-Axis",
                                      "1-Axis Backtracking", "2-Axis"])
        self.tilt_le = QtWidgets.QLineEdit("34")
        self.azimuth_le = QtWidgets.QLineEdit("180")
        self.rate_le = QtWidgets.QComboBox()
        self.rate_le.insertItems(0, ["Residential", "Commercial", "Industrial"])

        param_fields.addWidget(self.api_le)
        param_fields.addWidget(self.area_le)
        param_fields.addWidget(self.module_le)
        param_fields.addWidget(self.lat_le)
        param_fields.addWidget(self.lon_le)
        param_fields.addWidget(self.losses_le)
        param_fields.addWidget(self.array_le)
        param_fields.addWidget(self.tilt_le)
        param_fields.addWidget(self.azimuth_le)
        param_fields.addWidget(self.rate_le)

        output_layout = QtWidgets.QHBoxLayout()
        self.output_l = QtWidgets.QLabel("Save hourly data to file: ")
        self.output_check = QtWidgets.QCheckBox()
        self.savefilepath = QtWidgets.QLineEdit()
        self.fileselect = QtWidgets.QPushButton("Save as...")

        output_layout.addWidget(self.output_check)
        output_layout.addWidget(self.output_l)

        output_layout.addWidget(self.savefilepath)
        output_layout.addWidget(self.fileselect)

        self.submit = QtWidgets.QPushButton("Submit")

        parameters.addLayout(param_labels)
        parameters.addLayout(param_fields)

        left_pane.addLayout(parameters)
        left_pane.addWidget(self.submit)

        self.output_box = QtWidgets.QPlainTextEdit()
        self.output_box.setFixedWidth(500)

        right_pane.addWidget(self.output_box)
        right_pane.addLayout(output_layout)

        container.addLayout(left_pane)
        container.addLayout(right_pane)

        self.fileselect.clicked.connect(self.set_output_file)
        self.submit.clicked.connect(self.generate_output)
        self.setLayout(container)
        self.setWindowTitle("PVWatts Tool")
        self.show()

    def set_output_file(self):
        destination = str(QtWidgets.QFileDialog.getSaveFileName(self, 'Save CSV', os.getenv('HOME'), 'CSV(*.csv)')[0])
        self.savefilepath.setText(destination)

    def module_type(self):
        """Convert descriptive user-input into PVWatts API input parameters.

        PVWatts requires numeric input for the module type parameter; this method allows the user to select from
        more descriptive options and generate the required input parameter."""

        if self.module_le.currentText() == "Standard":
            moduletype_id = "0"
        elif self.module_le.currentText() == "Premium":
            moduletype_id = "1"
        elif self.module_le.currentText() == "Thin Film":
            moduletype_id = "2"
        return (moduletype_id)

    def array_type(self):
        """Convert descriptive user-input into PVWatts API input parameters.

        PVWatts requires numeric input for the array type parameter; this method allows the user to select from
        more descriptive options and generate the required input parameter."""

        if self.array_le.currentText() == "Fixed - Open Rack":
            arraytype_id = "0"
        elif self.array_le.currentText() == "Fixed - Roof Mounted":
            arraytype_id = "1"
        elif self.array_le.currentText() == "1-Axis":
            arraytype_id = "2"
        elif self.array_le.currentText() == "1-Axis Backtracking":
            arraytype_id = "3"
        elif self.array_le.currentText() == "2-Axis":
            arraytype_id = "4"
        return (arraytype_id)

    def generate_output(self):
        """Creates a global scenario object and prints descriptive information to the output box."""

        global scenario
        scenario = PVWatts_API.PVWatts_Run(api_key=self.api_le.text(),
                                           area=float(self.area_le.text()),
                                           module_type=int(self.module_type()),
                                           lat=self.lat_le.text(),
                                           lon=self.lon_le.text(),
                                           losses=self.losses_le.text(),
                                           array_type=self.array_type(),
                                           tilt=self.tilt_le.text(),
                                           azimuth=self.azimuth_le.text(),
                                           timeframe='hourly',
                                           ratetype=self.rate_le.currentText().lower())
        if self.first_run:
            self.output_box.appendPlainText(scenario.describe())
            self.first_run = False
        else:
            self.output_box.appendPlainText('--------------------' + '\n' + scenario.describe())

        if self.output_check.isChecked():
            scenario.hourly_data.to_csv(self.savefilepath.text())


def run():
    """Make GUI call-able from python interpreter.

    The user can initialize the GUI from a python interpreter by calling PVWatts_Tool.run()"""

    app = QtWidgets.QApplication(sys.argv)
    current_run = Window()
    sys.exit(app.exec())



from distutils.core import setup
from setuptools import find_packages

setup(
    name='PVWatts_Tool',
    version='1.0.3',
    url='https://github.com/warnuk/PVWatts_Tool',
    packages=['PVWatts_Tool'],
    license='MIT',
    author='warnuk',
    author_email='warnuk@umich.edu',
    description='A tool that calls on NREL APIs to generate useful information for solar PV resource planning.',
    install_requires = ['pandas', 'requests', 'PyQt5'],
    python_requires = '>=3',


)



""""nrel_requests.py
by warnuk

This module contains functions that take input parameters and call the PVWatts API and Utility Rates API."""

import requests

__author__ = "warnuk"
__credits__ = ["warnuk", "NREL", "PVWatts"]
__license__ = "MIT"
__version__ = "1.0.1"
__maintainer__ = "warnuk"
__email__ = "warnuk@umich.edu"
__status__ = "Development"

def pvwatts_output(system_capacity, module_type, api_key, **kwargs):
    """ Uses the PVWatts API to retrieve output from PVWatts API for given inputs."""

    # Define url and individual params seperately.
    url = "https://developer.nrel.gov/api/pvwatts/v5.json?"
    system_capacity = str(system_capacity)
    module_type = str(module_type)
    # Join params and url together to form request statement.
    params = "api_key=%s&system_capacity=%s&module_type=%s" %(api_key,system_capacity,module_type)
    for key, value in kwargs.items():
        params += "&%s=%s" % (key, value)
    params += "&format=JSON"
    request = url + params
    outputs = requests.get(request).json()['outputs']
    return(outputs)

def utility_output(lat, lon, api_key):
    """ utility_output uses the NREL Utility Rates API to retrieve
    utility info/rates for a given location"""

    url = "https://developer.nrel.gov/api/utility_rates/v3.json?"
    params = f"api_key={api_key}&lat={lat}&lon={lon}"
    request = url + params
    outputs = requests.get(request).json()['outputs']
    return(outputs)

